<?php

class RCPGA_Group_Capabilities {

	/**
	 * @var
	 */
	protected static $_instance;

	/**
	 * Only make one instance of the OM_Child_Members
	 *
	 * @return RCPGA_Group_Capabilities
	 */
	public static function get_instance() {
		if ( ! self::$_instance instanceof RCPGA_Group_Capabilities ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	protected function __construct() {

		// none of these filters should run in the admin
		if ( is_admin() && ! ( defined( 'DOING_AJAX' ) && DOING_AJAX ) ) {
			return;
		}

		if ( version_compare( RCP_PLUGIN_VERSION, '3.0', '>=' ) ) {
			// RCP version 3.0 and higher.
			add_filter( 'rcp_member_can_access', array( $this, 'member_can_access' ), 10, 4 );
			add_filter( 'rcp_restrict_shortcode_has_access', array( $this, 'member_restrict_shortcode_access' ), 10, 3 );
		}

		// Backwards compat for 2.9 and lower.
		add_filter( 'rcp_is_active',                  array( $this, 'group_is_active'       ), 10, 2 );
		add_filter( 'rcp_member_get_status',          array( $this, 'group_get_status'      ), 10, 2 );
		add_filter( 'rcp_member_get_subscription_id', array( $this, 'group_subscription_id' ), 10, 2 );
		add_filter( 'rcp_member_get_expiration_date', array( $this, 'group_expiration_date' ), 10, 5 );
		add_filter( 'rcp_get_joined_date',            array( $this, 'group_joined_date'     ), 10, 3 );
	}

	/**
	 * Supported roles
	 *
	 * @return array
	 */
	public function get_roles() {
		return array( 'owner', 'admin', 'member', 'invited' );
	}

	/**
	 * Supported tasks
	 *
	 * @return array
	 */
	public function get_tasks() {
		return array( 'manage_billing', 'manage_members', 'view_group' );
	}

	/**
	 * Determine which task can be performed by this role
	 *
	 * @param string $role
	 *
	 * @return array
	 */
	public function get_tasks_of_role( $role = '' ) {

		$tasks = array();

		if ( empty( $role ) ) {
			return $tasks;
		}

		switch ( $role ) {

			case 'owner' :

				$tasks[] = 'manage_billing';
				$tasks[] = 'manage_members';
				$tasks[] = 'view_group';

				break;

			case 'admin' :

				$tasks[] = 'manage_members';
				$tasks[] = 'view_group';

				break;

			case 'member' :

				$tasks[] = 'view_group';

		}

		return apply_filters( 'rcpga_get_tasks_of_role', $tasks, $role );

	}

	/**
	 * Determine if this user can perform this action
	 *
	 * @param string $task
	 * @param int    $user_id
	 * @param int    $group_id
	 *
	 * @return bool
	 */
	public function can( $task = '', $user_id = 0, $group_id = 0 ) {

		if ( empty( $task ) ) {
			return false;
		}

		if ( empty( $user_id ) ) {
			$user_id = get_current_user_id();
		}

		if ( user_can( $user_id, 'manage_options' ) ) {
			return true;
		}

		if ( empty( $group_id ) ) {
			$group_id = rcpga_group_accounts()->members->get_group_id( $user_id );
		}

		// Get the member's role in the group
		$role = rcpga_group_accounts()->members->get_role( $user_id );

		// Get the tasks their role has
		$tasks = $this->get_tasks_of_role( $role );

		// Check that this user is in the specified group
		if ( ! empty( $group_id ) ) {
			$in_group = (int) rcpga_group_accounts()->members->get_group_id( $user_id ) === (int) $group_id;
		} else {
			$in_group = true;
		}

		return in_array( $task, $tasks ) && $in_group;

	}

	/**
	 * Grants a group member access to view the post if the group owner has permission to view that post.
	 *
	 * @param bool       $can_access Whether or not the user can access the post.
	 * @param int        $user_id    ID of the user.
	 * @param int        $post_id    ID of the post being checked.
	 * @param RCP_Member $member     Member object.
	 *
	 * @since 1.4.1
	 * @return bool
	 */
	public function member_can_access( $can_access, $user_id, $post_id, $member ) {

		// This user is not in a group - bail.
		if ( ! $group_id = rcpga_group_accounts()->members->get_group_id( $user_id ) ) {
			return $can_access;
		}

		// This is a group owner - bail.
		if ( rcpga_group_accounts()->members->is_group_owner( $user_id ) ) {
			return $can_access;
		}

		// Ignore invited members
		if ( 'invited' == rcpga_group_accounts()->members->get_role( $user_id ) ) {
			return $can_access;
		}

		if ( ! $can_access ) {
			$group_owner_user_id  = rcpga_group_accounts()->groups->get_owner_id( $group_id );
			$group_owner_customer = rcp_get_customer_by_user_id( $group_owner_user_id );

			if ( ! empty( $group_owner_customer ) ) {
				$can_access = $group_owner_customer->can_access( $post_id );;
			}
		}

		return $can_access;

	}

	/**
	 * Grants a group member access to view content in the [restrict] shortcode if the group owner has permission
	 * to view that content.
	 *
	 * @param bool  $can_access Whether or not the user can view the content.
	 * @param int   $user_id    ID of the user being checked.
	 * @param array $atts       Shortcode attributes.
	 *
	 * @since 1.4.1
	 * @return bool
	 */
	public function member_restrict_shortcode_access( $can_access, $user_id, $atts ) {

		// This user is not in a group - bail.
		if ( ! $group_id = rcpga_group_accounts()->members->get_group_id( $user_id ) ) {
			return $can_access;
		}

		// This is a group owner - bail.
		if ( rcpga_group_accounts()->members->is_group_owner( $user_id ) ) {
			return $can_access;
		}

		// Ignore invited members
		if ( 'invited' == rcpga_group_accounts()->members->get_role( $user_id ) ) {
			return $can_access;
		}

		// If user can already view it, no need to proceed.
		if ( $can_access ) {
			return $can_access;
		}

		$group_owner_user_id  = rcpga_group_accounts()->groups->get_owner_id( $group_id );
		$group_owner_customer = rcp_get_customer_by_user_id( $group_owner_user_id );

		// No group owner customer record - bail.
		if ( empty( $group_owner_customer ) ) {
			return $can_access;
		}

		$subscriptions = array_map( 'trim', explode( ',', $atts['subscription'] ) );
		$is_active     = ! empty( $group_owner_customer ) ? $group_owner_customer->has_active_membership() : false;

		if( $atts['paid'] ) {

			if ( ! empty( $group_owner_customer ) && $group_owner_customer->has_paid_membership() && $group_owner_customer->has_access_level( $atts['level'] ) ) {
				$can_access = true;
			}

		} else {

			if ( ! empty( $group_owner_customer ) && $group_owner_customer->has_access_level( $atts['level'] ) ) {
				$can_access = true;
			}
		}

		if ( ! empty( $subscriptions ) && ! empty( $subscriptions[0] ) ) {
			if ( $is_active && count( array_intersect( rcp_get_customer_membership_level_ids( $group_owner_customer->get_id() ), $subscriptions ) ) ) {
				$can_access = true;
			} else {
				$can_access = false;
			}
		}

		if ( $atts['userlevel'] === 'none' && ! is_user_logged_in() ) {
			$can_access = false;
		}
		if( 'none' != $atts['userlevel'] ) {
			$roles = array_map( 'trim', explode( ',', $atts['userlevel'] ) );

			foreach ( $roles as $role ) {
				if ( current_user_can( strtolower( $role ) ) ) {
					$can_access = true;
					break;
				} else {
					$can_access = false;
				}
			}
		}

		// No access if pending email verification.
		if ( ! empty( $group_owner_customer ) && $group_owner_customer->is_pending_verification() ) {
			$can_access = false;
		}

		return $can_access;

	}

	/**
	 * Filter rcp_is_active to allow for active groups
	 *
	 * @param $ret
	 * @param $user_id
	 *
	 * @return bool
	 */
	public function group_is_active( $ret, $user_id ) {

		if ( ! $group_id = rcpga_group_accounts()->members->get_group_id( $user_id ) ) {
			return $ret;
		}

		// if this is the owner account, bale or we'll get an endless loop
		if ( rcpga_group_accounts()->members->is_group_owner( $user_id ) ) {
			return $ret;
		}

		// ignore invited members
		if ( 'invited' == rcpga_group_accounts()->members->get_role( $user_id ) ) {
			return $ret;
		}

		if ( ! $ret ) {
			$ret = rcpga_group_accounts()->groups->is_group_active( $group_id );
		}

		return apply_filters( 'rcpga_group_is_active', $ret, $group_id, $user_id );

	}

	/**
	 * Get group subscription id
	 *
	 * @param $subscription_id
	 * @param $user_id
	 *
	 * @return int|false
	 */
	public function group_subscription_id( $subscription_id, $user_id ) {

		if ( ! $group_id = rcpga_group_accounts()->members->get_group_id( $user_id ) ) {
			return $subscription_id;
		}

		// if this is the owner account, bale or we'll get an endless loop
		if ( rcpga_group_accounts()->members->is_group_owner( $user_id ) ) {
			return $subscription_id;
		}

		// ignore invited members
		if ( 'invited' == rcpga_group_accounts()->members->get_role( $user_id ) ) {
			return $subscription_id;
		}

		// get the parent subscription
		if ( version_compare( RCP_PLUGIN_VERSION, '3.0', '>=') ) {
			$customer = rcp_get_customer_by_user_id( rcpga_group_accounts()->groups->get_owner_id( $group_id ) );

			if ( ! empty( $customer ) ) {
				$membership = rcp_get_customer_single_membership( $customer->get_id() );

				if ( ! empty( $membership ) ) {
					$subscription_id = $membership->get_object_id();
				}
			}
		} else {
			$subscription_id = rcp_get_subscription_id( rcpga_group_accounts()->groups->get_owner_id( $group_id ) );
		}

		return apply_filters( 'rcpga_group_subscription_id', $subscription_id, $group_id, $user_id );

	}

	/**
	 * Get the group expiration date
	 *
	 * @param string     $exp_date  Default member's expiration date.
	 * @param int        $user_id   ID of the user being checked.
	 * @param RCP_Member $member    Member object.
	 * @param bool       $formatted Whether to return a formatted version of the date.
	 * @param bool       $pending   Whether to return the pending expiration date, if set.
	 *
	 * @return string
	 */
	public function group_expiration_date( $exp_date, $user_id, $member, $formatted = false, $pending = false ) {

		if ( ! $group_id = rcpga_group_accounts()->members->get_group_id( $user_id ) ) {
			return $exp_date;
		}

		// if this is the owner account, bale or we'll get an endless loop
		if ( rcpga_group_accounts()->members->is_group_owner( $user_id ) ) {
			return $exp_date;
		}

		// ignore invited members
		if ( 'invited' == rcpga_group_accounts()->members->get_role( $user_id ) ) {
			return $exp_date;
		}

		if ( version_compare( RCP_PLUGIN_VERSION, '3.0', '>=') ) {
			$customer = rcp_get_customer_by_user_id( rcpga_group_accounts()->groups->get_owner_id( $group_id ) );

			if ( ! empty( $customer ) ) {
				$membership = rcp_get_customer_single_membership( $customer->get_id() );

				if ( ! empty( $membership ) ) {
					$exp_date = $membership->get_expiration_date( $formatted );
				}
			}
		} else {
			$owner    = new RCP_Member( rcpga_group_accounts()->groups->get_owner_id( $group_id ) );
			$exp_date = $owner->get_expiration_date( $formatted, $pending );
		}

		return apply_filters( 'rcpga_group_expiration_date', $exp_date, $group_id, $user_id, $formatted, $pending );

	}

	/**
	 * Get the group status
	 *
	 * @param $status
	 * @param $user_id
	 *
	 * @return string
	 */
	public function group_get_status( $status, $user_id ) {

		if ( ! $group_id = rcpga_group_accounts()->members->get_group_id( $user_id ) ) {
			return $status;
		}

		// if this is the owner account, bale or we'll get an endless loop
		if ( rcpga_group_accounts()->members->is_group_owner( $user_id ) ) {
			return $status;
		}

		// ignore invited members
		if ( 'invited' == rcpga_group_accounts()->members->get_role( $user_id ) ) {
			return $status;
		}

		// get the group status
		if ( version_compare( RCP_PLUGIN_VERSION, '3.0', '>=') ) {
			$customer = rcp_get_customer_by_user_id( rcpga_group_accounts()->groups->get_owner_id( $group_id ) );

			if ( ! empty( $customer ) ) {
				$membership = rcp_get_customer_single_membership( $customer->get_id() );

				if ( ! empty( $membership ) ) {
					$status = $membership->get_status();
				}
			}
		} else {
			$status = rcp_get_status( rcpga_group_accounts()->groups->get_owner_id( $group_id ) );
		}

		return apply_filters( 'rcpga_group_status', $status, $group_id, $user_id );
	}

	/**
	 * Gets the group's joined date for the specified subscription level ID.
	 *
	 * @see RCP_Member::get_joined_date()
	 * @param string $date            The member's joined date.
	 * @param int    $member_id       The member ID.
	 * @param int    $subscription_id The member's subscription level ID.
	 *
	 * @since 1.3.2
	 * @return string
	 */
	public function group_joined_date( $date, $member_id, $subscription_id ) {

		if ( ! $group_id = rcpga_group_accounts()->members->get_group_id( $member_id ) ) {
			return $date;
		}

		if ( rcpga_group_accounts()->members->is_group_owner( $member_id ) ) {
			return $date;
		}

		$owner_id = rcpga_group_accounts()->groups->get_owner_id( $group_id );

		if ( version_compare( RCP_PLUGIN_VERSION, '3.0', '>=') ) {
			$customer = rcp_get_customer_by_user_id( $owner_id );

			if ( ! empty( $customer ) ) {
				$membership = rcp_get_customer_single_membership( $customer->get_id() );

				if ( ! empty( $membership ) ) {
					$date = $membership->get_created_date( false );
				}
			}
		} else {
			$owner = new RCP_Member( $owner_id );
			$date  = $owner->get_joined_date( $subscription_id );
		}

		return $date;
	}

}