<?php

class RCPGA_Groups extends RCPGA_Groups_DB {


	/**
	 * Only make one instance of the OM_Child_Members
	 *
	 * @return RCPGA_Groups
	 */
	public static function get_instance() {
		if ( ! self::$_instance instanceof RCPGA_Groups ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	/**
	 * Get things started
	 *
	 * @access  public
	 * @since   1.0
	 */
	protected function __construct() {
		global $wpdb;
		$this->db_group    = 'groups';
		$this->table_name  = $wpdb->prefix . 'rcp_groups';
		$this->primary_key = 'group_id';
		$this->version     = '0.2';

		add_action( 'admin_init', array( $this, 'upgrades' ), -999 );
	}

	/**
	 * Get table columns and data types
	 *
	 * @access  public
	 * @since   1.0
	 */
	public function get_columns() {
		return array(
			'group_id'      => '%d',
			'owner_id'      => '%d',
			'membership_id' => '%d',
			'name'          => '%s',
			'description'   => '%s',
			'member_count'  => '%d',
			'seats'         => '%d',
			'date_created'  => '%s',
		);
	}

	/**
	 * Determines if the group is active.
	 *
	 * A group is active if the group owner has an active subscription
	 *
	 * @param int $group_id
	 * @access  public
	 * @since   1.0
	 * @return  bool
	 */
	public function is_group_active( $group_id = 0 ) {

		$owner_id = $this->get_owner_id( $group_id );
		$active   = false;

		if ( version_compare( RCP_PLUGIN_VERSION, '3.0', '>=') ) {
			// RCP 3.0+
			$membership_id = $this->get_membership_id( $group_id );

			if ( ! empty( $membership_id ) ) {
				$membership = rcp_get_membership( $membership_id );
			} else {
				$membership = false;
				$customer   = rcp_get_customer_by_user_id( $owner_id );

				if ( ! empty( $customer ) ) {
					$membership = rcp_get_customer_single_membership( $customer->get_id() );
				}
			}

			if ( ! empty( $membership ) ) {
				$active = $membership->is_active() && $membership->is_paid();
			}
		} else {
			// Backwards compat for 2.9 and earlier.
			if ( ! empty( $owner_id ) ) {
				$active = rcp_is_active( $owner_id );
			}
		}

		return apply_filters( 'rcpga_is_group_active', $active, $group_id );

	}

	/**
	 * Get the group name
	 *
	 * @param   int $group_id
	 * @access  public
	 * @since   1.0
	 * @return  string
	 */
	public function get_name( $group_id = 0 ) {
		return $this->get_column( 'name', $group_id );
	}

	/**
	 * Get the group description
	 *
	 * @param   int $group_id
	 * @access  public
	 * @since   1.0
	 * @return  string
	 */
	public function get_description( $group_id = 0 ) {
		return $this->get_column( 'description', $group_id );
	}

	/**
	 * Get the group member_count
	 *
	 * @param   int $group_id
	 * @access  public
	 * @since   1.0
	 * @return  int
	 */
	public function get_member_count( $group_id = 0 ) {
		return absint( $this->get_column( 'member_count', $group_id ) );
	}

	/**
	 * Get the number of seats
	 *
	 * @param   int $group_id
	 * @access  public
	 * @since   1.0
	 * @return  int
	 */
	public function get_seats_count( $group_id = 0 ) {
		return absint( $this->get_column( 'seats', $group_id ) );
	}

	/**
	 * Get the group owner_id
	 *
	 * @param   int $group_id
	 * @access  public
	 * @since   1.0
	 * @return  int
	 */
	public function get_owner_id( $group_id = 0 ) {
		return $this->get_column( 'owner_id', $group_id );
	}

	/**
	 * Get the ID of the corresponding membership record
	 *
	 * @param int $group_id ID of teh group.
	 *
	 * @access public
	 * @since 1.4.1
	 * @return int
	 */
	public function get_membership_id( $group_id = 0 ) {
		$membership_id = $this->get_column( 'membership_id', $group_id );

		// Backfill membership ID.
		if ( empty( $membership_id ) && version_compare( RCP_PLUGIN_VERSION, '3.0', '>=' ) ) {
			$owner_id = $this->get_owner_id( $group_id );
			$customer = rcp_get_customer_by_user_id( $owner_id );

			if ( ! empty( $customer ) ) {
				$membership = rcp_get_customer_single_membership( $customer->get_id() );

				if ( ! empty( $membership ) ) {
					$membership_id = $membership->get_id();

					$this->update( $group_id, array( 'membership_id' => absint( $membership_id ) ) );
				}
			}
		}

		return $membership_id;
	}

	/**
	 * Get group by owner id
	 *
	 * @param $owner_id
	 *
	 * @return object|false
	 */
	public function get_group_by_owner( $owner_id ) {
		return $this->get_by( 'owner_id', absint( $owner_id ) );
	}

	/**
	 * Return group role
	 *
	 * @param $group_id
	 *
	 * @return bool
	 */
	public function get_group_role( $group_id ) {

		if ( ! $owner_id = $this->get_owner_id( $group_id ) ) {
			return false;
		}

		if ( version_compare( RCP_PLUGIN_VERSION, '3.0', '>=' ) ) {
			$level_id      = false;
			$membership_id = $this->get_membership_id( $group_id );
			$membership    = false;

			if ( ! empty( $membership_id ) ) {
				$membership = rcp_get_membership( $membership_id );
			} else {
				$customer = rcp_get_customer_by_user_id( $owner_id );

				if ( ! empty( $customer ) ) {
					$membership = rcp_get_customer_single_membership( $customer->get_id() );
				}
			}

			if ( ! empty( $membership ) ) {
				$level_id = $membership->get_object_id();
			}
		} else {
			$level_id = rcp_get_subscription_id( $owner_id );
		}

		if ( empty( $level_id ) ) {
			return false;
		}

		if ( ! $level = rcp_get_subscription_details( $level_id ) ) {
			return false;
		}

		return $level->role;
	}

	/**
	 * Retrieve groups from the database
	 *
	 * @access  public
	 * @since   1.0
	 */
	public function get_groups( $args = array() ) {
		global $wpdb;

		$defaults = array(
			'number'  => 20,
			'offset'  => 0,
			'status'  => '',
			'order'   => 'DESC',
			'orderby' => 'group_id',
			'where'   => '',
		);

		$args = wp_parse_args( $args, $defaults );

		if ( $args['number'] < 1 ) {
			$args['number'] = 999999999999;
		}

		$where  = $args['where'];
		$key    = sprintf( 'get_groups:%s:%s', md5( serialize( $args ) ), $this->get_last_changed() );
		$groups = wp_cache_get( $key, $this->table_name );

		if ( false === $groups ) {
			$groups = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$this->table_name} {$where} ORDER BY {$args['orderby']} {$args['order']} LIMIT %d,%d;", absint( $args['offset'] ), absint( $args['number'] ) ) );
			wp_cache_set( $key, $groups, $this->table_name );
		}

		return apply_filters( 'rcpga_get_groups', $groups, $args );

	}

	/**
	 * Retrieve count of groups from database
	 *
	 * @access  public
	 * @since   1.0.1
	 */
	public function get_group_count() {
		global $wpdb;

		$key         = sprintf( 'get_group_count:%s', $this->get_last_changed() );
		$group_count = wp_cache_get( $key, $this->table_name );

		if ( false === $group_count ) {
			$group_count = $wpdb->get_results( "SELECT COUNT(*) AS group_count FROM {$this->table_name};" );
			$group_count = absint( $group_count[0]->group_count );

			wp_cache_set( $key, $group_count, $this->table_name );
		}

		return apply_filters( 'rcpga_get_group_count', $group_count );

	}

	/**
	 * Adds a new group
	 *
	 * @param   array $args
	 * @access  public
	 * @since   1.0
	 * @return  int|false
	 */
	public function add( $args = array() ) {

		$defaults = array(
			'owner_id'      => 0,
			'membership_id' => 0,
			'description'   => '',
			'name'          => '',
			'seats'         => 0,
			'member_count'  => 0,
			'date_created'  => current_time( 'mysql' ),
		);

		$args = wp_parse_args( $args, $defaults );

		if ( empty( $args['owner_id'] ) ) {
			return false;
		}

		return $this->insert( $args, 'group' );

	}

	/**
	 * Update the group's member count
	 *
	 * @param   int $group_id
	 * @access  public
	 * @since   1.0
	 * @return  int New count
	 */
	public function update_count( $group_id = 0 ) {

		if ( empty( $group_id ) ) {
			return;
		}

		$member_count = rcpga_group_accounts()->members->count( $group_id );

		$this->update( $group_id, array( 'member_count' => absint( $member_count ) ) );

		return $member_count;

	}

	/**
	 * Perform database upgrades.
	 *
	 * @access public
	 * @since 1.4.1
	 * @return void
	 */
	public function upgrades() {

		global $wpdb;

		$upgraded = false;

		$db_version = get_option( $this->table_name . '_db_version', $this->version );

		/**
		 * Version 0.2 upgrades.
		 */
		if ( version_compare( $db_version, '0.2', '<' ) ) {

			rcp_log( sprintf( 'Group Accounts: Current groups database version is: %s. Performing 0.2 upgrades.', $db_version ), true );

			// Add membership_id column.
			$group_cols              = $wpdb->get_col( "DESC " . $this->table_name, 0 );
			$membership_id_col_added = in_array( 'membership_id', $group_cols );

			if ( ! $membership_id_col_added ) {
				rcp_log( sprintf( 'Group Accounts: Adding membership_id column to table %s.', $this->table_name ), true );

				$membership_id_col_added = $wpdb->query( "ALTER TABLE {$this->table_name} ADD membership_id bigint(20) UNSIGNED NOT NULL AFTER owner_id" );

				if ( false === $membership_id_col_added ) {
					rcp_log( sprintf( 'Group Accounts: Error adding the membership_id column to table %s.', $this->table_name ), true );
				} else {
					$upgraded = true;
				}
			} else {
				rcp_log( sprintf( 'Group Accounts: membership_id column is already present in table %s.', $this->table_name ), true );
			}

			// Drop owner_id key and re-add as non-unique.
			rcp_log( sprintf( 'Group Accounts: Dropping unique owner_id key from table %s.', $this->table_name ), true );
			$dropped = $wpdb->query( "ALTER TABLE {$this->table_name} DROP INDEX owner_id" );

			if ( $dropped ) {
				rcp_log( sprintf( 'Group Accounts: Adding non-unique owner_id key to table %s.', $this->table_name ), true );
				$wpdb->query( "ALTER TABLE {$this->table_name} ADD KEY owner_id (owner_id)" );

				$upgraded = true;
			}

		}

		if ( $upgraded ) {
			// Update DB version.
			rcp_log( sprintf( 'Group Accounts: Updating groups database version to %s.', $this->version ), true );
			update_option( $this->table_name . '_db_version', $this->version );
		}

	}

	/**
	 * Create the table
	 *
	 * @access  public
	 * @since   1.0
	 */
	public function create_table() {
		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

		global $wpdb;

		// Check if table exists.
		$query    = "SHOW TABLES LIKE %s";
		$like     = $wpdb->esc_like( $this->table_name );
		$prepared = $wpdb->prepare( $query, $like );
		$result   = $wpdb->get_var( $prepared );

		if ( ! empty( $result ) && ! is_wp_error( $result ) ) {
			// Try running upgrades first. This prevents annoying duplicate key errors with dbDelta().
			$this->upgrades();
		}

		$sql = "CREATE TABLE {$this->table_name} (
			`group_id` bigint(20) NOT NULL AUTO_INCREMENT,
			`owner_id` bigint(20) NOT NULL,
			`membership_id` bigint(20) UNSIGNED NOT NULL,
			`name` mediumtext NOT NULL,
			`description` longtext NOT NULL,
			`seats` bigint(20) NOT NULL,
			`member_count` bigint(20) NOT NULL,
			`date_created` datetime NOT NULL,
			PRIMARY KEY (group_id),
			KEY owner_id (owner_id)
			) CHARACTER SET utf8 COLLATE utf8_general_ci;";

		dbDelta( $sql );

		update_option( $this->table_name . '_db_version', $this->version );
	}


}
